--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]
-- Layout Settings
local string_find = string.find
local layout = {
    mainFrame = {
        "CENTER",
        "UIParent",
        "CENTER",
        0,
        0,
        512,
        256
    },
    -- Frames: 2 main frames to contain the bars and text for each of the sides
    frames = {
        leftFrame = {
            "LEFT",
            "mainFrame",
            "LEFT",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\bg_21pp",
        },
        rightFrame = {
            "RIGHT", 
            "mainFrame",
            "RIGHT",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\bg_21",
        }
    },
    
    -- Bars: 2 sets of bars, one for each side. Each set contains 3 main bars, 2 pet bars, 1 casting and 1 flash, this allows the user to choose different layouts.
    bars = {
        li = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            0,          -- x
            0,          -- y
            128,        -- width
            256,        -- height
            0,          -- x0
            1,          -- x1
            0,          -- y0
            1,          -- y1
            "Interface\\AddOns\\DHUD2\\textures\\1",
            256,        -- texture height
            11,         -- texture gap top
            11          -- texture gap bottom
        },
        lm = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\2",
            256,
            5,
            5
        },
        lo = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\3",
            256,
            5,
            5
        },
        lpo = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\p1",
            256,
            128,
            20            
        },
        lpi = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\p2",
            256,
            128,
            20            
        },
        lc = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\cb",
            256,
            11,
            11
        },
        lcf = {
            "BOTTOM",
            "lc",
            "BOTTOM",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\cbh",
            256,
            11,
            11
        },
    
        ri = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\1",
            256,
            11,
            11
        },
        rm = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\2",
            256,
            5,
            5
        },
        ro = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\3",
            256,
            5,
            5
        },
        rpo = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\p1",
            256,
            128,
            20            
        },
        rpi = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\p2",
            256,
            128,
            20            
        },
        rc = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\cb",
            256,
            11,
            11
        },
        rcf = {
            "BOTTOM",
            "rc",
            "BOTTOM",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\cbh",
            256,
            11,
            11
        },
    },
    
    -- Texts: Each bar can have an asociated text to display it's value.
    barText = {
        li = {
            "TOPLEFT",
            "leftFrame",
            "BOTTOMRIGHT",
            -25,
            10,
            200,
            14,
            "LEFT",
        },
        lm = {
            "TOPLEFT",
            "leftFrame",
            "BOTTOMRIGHT",
            -40,
            -5,
            200,
            14,
            "LEFT",
        },
        lo = {
            "TOPRIGHT",
            "leftFrame",
            "BOTTOMRIGHT",
            -80,
            0,
            200,
            14,
            "RIGHT",
        },        
        lpo = {
            "TOPLEFT",
            "leftFrame",
            "BOTTOMRIGHT",
            0,
            25,
            200,
            14,
            "LEFT",
        },
        lpi = {
            "TOPLEFT",
            "leftFrame",
            "BOTTOMRIGHT",
            0,
            40,
            200,
            14,
            "LEFT",
        },
        lc = {
            "TOPLEFT",
            "lc",
            "TOPRIGHT",
            -22,
            0,
            100,
            14,
            "RIGHT",
        },
        lcd = {
            "TOPLEFT",
            "lc",
            "TOPRIGHT",
            -22,
            15,
            100,
            14,
            "RIGHT",
        },
        
        ri = {
            "TOPRIGHT",
            "rightFrame",
            "BOTTOMLEFT",
            15,
            10,
            200,
            14,
            "RIGHT",
        },
        rm = {
            "TOPRIGHT",
            "rightFrame",
            "BOTTOMLEFT",
            30,
            -5,
            200,
            14,
            "RIGHT",
        },
        ro = {
            "TOPLEFT",
            "rightFrame",
            "BOTTOMLEFT",
            70,
            0,
            200,
            14,
            "LEFT",
        },
        rpo = {
            "TOPRIGHT",
            "rightFrame",
            "BOTTOMLEFT",
            -10,
            25,
            200,
            14,
            "RIGHT",
        },
        rpi = {
            "TOPRIGHT",
            "rightFrame",
            "BOTTOMLEFT",
            -10,
            40,
            200,
            14,
            "RIGHT",
        },
        rc = {
            "TOPRIGHT",
            "rc",
            "TOPLEFT",
            0,
            0,
            100,
            14,
            "RIGHT",
        },
        rcd = {
            "TOPRIGHT",
            "rc",
            "TOPLEFT",
            0,
            15,
            100,
            14,
            "RIGHT",
        },
    },
    
    texts = {
        target= {
            "CENTER",
            "mainFrame",
            "CENTER",
            0,
            -160,
            200,
            14,
        },
        targetTarget = {
            "CENTER",
            "mainFrame",
            "CENTER",
            0,
            -175,
            200,
            14,
        },
    },
    
    buttons = {
        b1 = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            6-16,
            0,
            20,
            20,
        },
        b2 = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            -1-16,
            20,
            20,
            20,
        },
        b3 = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            -7-16,
            40,
            20,
            20,
        },
        b4 = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            -10-16, 
            60, 
            20, 
            20,
        },
        b5 = {
            "BOTTOM",
            "leftFrame",
            "BOTTOM",
            -12-16, 
            80, 
            20, 
            20,
        },
    },
    
    icons = {
        targetIcon = {
            "BOTTOM",
            "target_Text",
            "TOP",
            0,
            0, 
            25, 
            25,
            0.6,
            0,
            0,
            0.6,
            {
                "Interface\\TargetingFrame\\UI-PVP-Alliance",
                "Interface\\TargetingFrame\\UI-PVP-Horde",
                "Interface\\TargetingFrame\\UI-PVP-FFA",                
            }
        },
        playerResting = {
            "TOP",
            "leftFrame",
            "TOP", 
            42, 
            12,
            25,
            25,
            0.0625,
            0.4475,
            0.0625,
            0.4375,
            "Interface\\CharacterFrame\\UI-StateIcon"
        },
        playerInCombat = {
            "TOP",
            "leftFrame", 
            "TOP",
            42, 
            12, 
            25, 
            25,
            0.5625,
            0.9375,
            0.0625,
            0.4375,
            "Interface\\CharacterFrame\\UI-StateIcon"
        },
        playerLeader = {
            "TOP",
            "leftFrame",
            "TOP", 
            48, 
            -20, 
            20, 
            20,
            0,
            1,
            0,
            1,
            "Interface\\GroupFrame\\UI-Group-LeaderIcon"
        },
        playerLooter = {
            "TOP",
            "leftFrame",
            "TOP",
            43,
            -40,
            18,
            18,
             0,
            1,
            0,
            1,
            "Interface\\GroupFrame\\UI-Group-MasterLooter"
        },
        playerPvP = {
            "TOP",
            "leftFrame",
            "TOP",
            50,
            -15,
            25,
            25,
            0,
            0.6,
            0,
            0.6,
            {
                "Interface\\TargetingFrame\\UI-PVP-Alliance",
                "Interface\\TargetingFrame\\UI-PVP-Horde",
                "Interface\\TargetingFrame\\UI-PVP-FFA",
            }
        },
        petHappy = {
            "TOP",
            "leftFrame",
            "TOP",
            36,
            -107,
            20,
            20,
            0,
            0.1875,
            0,
            0.359375,
            "Interface\\PetPaperDollFrame\\UI-PetHappiness"
        },
        targetElite = {
            "TOP",
            "leftFrame",
            "TOP",
            18,
            20,
            64,
            64,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD2\\textures\\elite"           
        },
        --[[
        raidIcon = {  
            "TOP",
            "leftFrame",
            "TOP",
            48,
            -43,
            25,
            25,
            0.75,
            0.75,
            0.75,
            0.75,
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons"
        },
        ]]
    },
    
    targetRightAuras = {
        TRA1 = {
            "RIGHT",
            "target_Text",
            "LEFT",
            -1,
            0,
            20, 
            20
        },
        TRA2 = {
            "RIGHT",
            "TRA1" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA3 = {
            "RIGHT",
            "TRA2" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA4 = {
            "RIGHT",
            "TRA3" ,
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA5 = {
            "RIGHT",
            "TRA4" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA6 = {
            "RIGHT",
            "TRA5" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA7 = {
            "RIGHT",
            "TRA6" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA8 = {
            "RIGHT",
            "TRA7" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA9 = {
            "RIGHT",
            "TRA1" ,
            "LEFT",
            20,
            -21,
            20,
            20 
        },
        TRA10 = {
            "RIGHT",
            "TRA9" ,
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA11 = {
            "RIGHT",
            "TRA10",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA12 = {
            "RIGHT",
            "TRA11",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA13 = {
            "RIGHT",
            "TRA12",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA14 = {
            "RIGHT",
            "TRA13",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA15 = {
            "RIGHT",
            "TRA14",
            "LEFT",
            -1,
            0,
            20,
            20,
        },
        TRA16 = {
            "RIGHT",
            "TRA15",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA17 = {
            "RIGHT",
            "TRA9" ,
            "LEFT",
            20,
            -21,
            20,
            20
        },
        TRA18 = {
            "RIGHT",
            "TRA17",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA19 = {
            "RIGHT",
            "TRA18",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA20 = {
            "RIGHT",
            "TRA19",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA21 = {
            "RIGHT",
            "TRA20",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA22 = {  
            "RIGHT",
            "TRA21",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA23 = {
            "RIGHT",
            "TRA22",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA24 = {
            "RIGHT",
            "TRA23",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA25 = {
            "RIGHT",
            "TRA17",
            "LEFT",
            20,
            -21,
            20,
            20
        },
        TRA26 = {  
            "RIGHT",
            "TRA25",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA27 = {
            "RIGHT",
            "TRA26",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA28 = {
            "RIGHT",
            "TRA27",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA29 = {
            "RIGHT",
            "TRA28",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA30 = {
            "RIGHT",
            "TRA29",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA31 = {
            "RIGHT",
            "TRA30",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA32 = {
            "RIGHT",
            "TRA31",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA33 = {
            "RIGHT",
            "TRA25",
            "LEFT",
            20,
            -21,
            20,
            20
        },
        TRA34 = {
            "RIGHT",
            "TRA33",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA35 = {
            "RIGHT",
            "TRA34",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA36 = {
            "RIGHT",
            "TRA35",
            "LEFT",
            -1,
            0,
            20,
            20
        },
        TRA37 = {
            "RIGHT",
            "TRA36",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA38 = {
            "RIGHT",
            "TRA37",
            "LEFT",
            -1,
            0,
            20,
            20 
        },
        TRA39 = {
            "RIGHT",
            "TRA38",
            "LEFT",
            -1,
            0,
            20,
            20,
        },
        TRA40 = {
            "RIGHT",
            "TRA39",
            "LEFT",
            -1,
            0,
            20,
            20
        },
    },
    
    targetLeftAuras = {
        TLA1 = {
            "LEFT",
            "target_Text",
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA2 = {
            "LEFT",
            "TLA1"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA3 = {
            "LEFT",
            "TLA2"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA4 = {
            "LEFT",
            "TLA3"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA5 = {
            "LEFT",
            "TLA4"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA6 = {
            "LEFT",
            "TLA5"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA7 = {
            "LEFT",
            "TLA6"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA8 = {
            "LEFT",
            "TLA7"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA9 = {
            "LEFT",
            "TLA1"  ,
            "RIGHT" ,
            -20,
            -21,
            20,
            20
        },
        TLA10 = {
            "LEFT",
            "TLA9"  ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA11 = {
            "LEFT",
            "TLA10" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA12 = {
            "LEFT",
            "TLA11" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA13 = {
            "LEFT",
            "TLA12" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA14 = {
            "LEFT",
            "TLA13" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA15 = {
            "LEFT",
            "TLA14" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA16 = {
            "LEFT",
            "TLA15" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA17 = {
            "LEFT",
            "TLA9"  ,
            "RIGHT" ,
            -20,
            -21,
            20,
            20
        },
        TLA18 = {
            "LEFT",
            "TLA17" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA19 = {
            "LEFT",
            "TLA18" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA20 = {
            "LEFT",
            "TLA19" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA21 = {
            "LEFT",
            "TLA20" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA22 = {
            "LEFT",
            "TLA21" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA23 = {
            "LEFT",
            "TLA22" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA24 = {
            "LEFT",
            "TLA23" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA25 = {
            "LEFT",
            "TLA17" ,
            "RIGHT" ,
            -20,
            -21,
            20,
            20
        },
        TLA26 = {
            "LEFT",
            "TLA25" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA27 = {
            "LEFT",
            "TLA26" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA28 = {
            "LEFT",
            "TLA27" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA29 = {
            "LEFT",
            "TLA28" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA30 = {
            "LEFT",
            "TLA29" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA31 = {
            "LEFT",
            "TLA30" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA32 = {
            "LEFT",
            "TLA31" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA33 = {
            "LEFT",
            "TLA25" ,
            "RIGHT" ,
            -20,
            -21,
            20,
            20
        },
        TLA34 = {
            "LEFT",
            "TLA33" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA35 = {
            "LEFT",
            "TLA34" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA36 = {
            "LEFT",
            "TLA35" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA37 = {
            "LEFT",
            "TLA36" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA38 = {
            "LEFT",
            "TLA37" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA39 = {
            "LEFT",
            "TLA38" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
        TLA40 = {
            "LEFT",
            "TLA39" ,
            "RIGHT" ,
            1,
            0,
            20,
            20
        },
    },  
    
    playbuff = {
        playerBuff1 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            7+16,
            0,
            26,
            26
        },
        playerBuff2 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            16+16,
            31+1,
            26,
            26
        },
        playerBuff3 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            21+16,
            63+2,
            26,
            26
        },
        playerBuff4 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            24+16,
            96+3,
            26,
            26
        },
        playerBuff5 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            24+16,
            128+4,
            26,
            26
        },
        playerBuff6 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            22+16,
            162+5,
            26,
            26
        },
        playerBuff7 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            17+16,
            194+6,
            26,
            26
        },
        playerBuff8 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            9+16,
            226+7,
            26,
            26
        },
        playerBuff9 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            40+16,
            7+1,
            26,
            26
        },
        playerBuff10 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            47+16,
            42+2,
            26,
            26
        },
        playerBuff11 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            52+16,
            76+3,
            26,
            26
        },
        playerBuff12 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            53+16,
            111+4,
            26,
            26
        },
        playerBuff13 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            52+16,
            146+5,
            26,
            26
        },
        playerBuff14 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            49+16,
            182+6,
            26,
            26
        },
        playerBuff15 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            42+16,
            217+7,
            26,
            26
        },
        playerBuff16 = {
            "BOTTOM",
            "rightFrame",
            "BOTTOM",
            35+16,
            250+8,
            26,
            26    
        }
    },
    
    images = {
        pet = {
            {   
                "unHappy",
                "Interface\\PetPaperDollFrame\\UI-PetHappiness",
                0.375,
                0.5625,
                0,
                0.359375
            },
            { 
                "normal",
                "Interface\\PetPaperDollFrame\\UI-PetHappiness",
                0.1875,
                0.375,
                0,
                0.359375
            },
            {
                "happy",
                "Interface\\PetPaperDollFrame\\UI-PetHappiness",
                0,
                0.1875,
                0,
                0.359375               
            }            
        }
    },
    
    raidIcons = {
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0,
            0.25,
            0,
            0.25
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0.25,
            0.50,
            0,
            0.25
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0.50,
            0.75,
            0,
            0.25
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0.75,
            1,
            0,
            0.25
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0,
            0.25,
            0.25,
            0.50
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0.25,
            0.50,
            0.25,
            0.50
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0.50,
            0.75,
            0.25,
            0.50
        },
        {
            "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
            0.75,
            1,
            0.25,
            0.50
        },
    },
    defaultfont     = "Fonts/FRIZQT__.TTF",
    defaultfont_num = "Interface\\AddOns\\DHUD2\\textures\\Number.TTF",
}
DHUD2.layout = layout

function DHUD2:createFrames()
    self:Debug("createFrames")
    -- Get layout information
    local mainFrame = layout.mainFrame
    local subFrames = layout.frames
    local buttons = layout.buttons
    local icons = layout.icons
    local bars = layout.bars
    local barText = layout.barText
    local texts = layout.texts
    local tRA = layout.targetRightAuras
    local tLA = layout.targetLeftAuras
    local playBuff = layout.playbuff
  
    -- Main Frame
    local point, parent, relative, x, y, width, height = unpack(mainFrame)
    ref = CreateFrame ("Frame", "DHUD2_mainFrame", _G[parent])
    ref:ClearAllPoints()
    ref:SetPoint(point, parent, relative, x, y)
    ref:SetHeight(height)
    ref:SetWidth(width)
    ref:EnableMouse(false)
    ref:Show()
    
    -- Left and right frames were the bars and texts are placed
    for k, v in pairs(subFrames) do
        point, parent, relative, x, y, width, height, x0, x1, y0, y1, path = unpack(v)
        ref = CreateFrame("Frame", "DHUD2_"..k, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        local bgt = ref:CreateTexture("DHUD2_"..k.."_Texture", "BACKGROUND")
        bgt:SetTexture(path)
        bgt:ClearAllPoints()
        bgt:SetPoint("TOPLEFT", ref , "TOPLEFT", 0, 0)
        bgt:SetPoint("BOTTOMRIGHT", ref , "BOTTOMRIGHT", 0, 0)
        bgt:SetTexCoord(x0, x1, y0, y1)
        ref:EnableMouse(false)
    end
    
    -- Bars
    for k, v in pairs(bars) do
        point, parent, relative, x, y, width, height, x0, x1, y0, y1, path = unpack(v)
        ref = CreateFrame("Button", "DHUD2_"..k, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        bgt = ref:CreateTexture("DHUD2_"..k.."_Texture", "ARTWORK")
        bgt:SetTexture(path)
        bgt:SetPoint(point, ref, relative, 0, 0)
        bgt:SetHeight(height)
        bgt:SetWidth(width)
        bgt:SetTexCoord(x0, x1, y0, y1)
        bgt:SetBlendMode("BLEND")
        ref:EnableMouse(false)
        ref:Hide()
    end

    -- Bar's Text
    for k, v in pairs(barText) do
        point, parent, relative, x, y, width, height, justify = unpack(v)
        ref = CreateFrame("Button", "DHUD2_"..k.."_Text", _G["DHUD2_"..parent])
        ref:SetWidth(width)
        ref:SetHeight(height)
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        local font = ref:CreateFontString("DHUD2_"..k.."_Text_Text", "OVERLAY")
        font:SetFontObject(GameFontHighlightSmall)
        --font:SetHeight(height)
        font:ClearAllPoints()
        font:SetPoint("TOPLEFT", ref, "TOPLEFT", 0, 0)
        font:SetText(k.."2.5")
        font:SetWidth(font:GetStringWidth()+10)
        font:SetJustifyH(justify)
        font:Show()
        ref:SetHeight(height)
        ref:SetWidth(font:GetStringWidth())
        ref:Hide()
    end

    -- Text
    for k, v in pairs(texts) do
        point, parent, relative, x, y, width, height = unpack(v)
        ref = CreateFrame("Button", "DHUD2_"..k.."_Text", _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        local font = ref:CreateFontString("DHUD2_"..k.."_Text_Text", "OVERLAY")
        font:SetFontObject(GameFontHighlightSmall)
        font:ClearAllPoints()
        font:SetPoint("TOPLEFT", ref, "TOPLEFT", 0, 0)
        font:SetText(k)
        font:SetJustifyH("CENTER")
        font:SetWidth(font:GetStringWidth())
        font:SetHeight(height)
        font:Show()
        
        ref:SetHeight(height)
        ref:SetWidth(font:GetStringWidth())
        ref:Hide()
    end
    
    -- Buttons for Combos/Totems
    for k, v in pairs(buttons) do
        point, parent, relative, x, y, width, height = unpack(v)
        ref = CreateFrame("Button", "DHUD2_"..k, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        ref:SetNormalTexture("Interface\\AddOns\\DHUD2\\textures\\c1")
        bgt = ref:CreateTexture("DHUD2_"..k.."_Border", "OVERLAY")
        bgt:SetTexture("Interface\\AddOns\\DHUD2\\textures\\serenity0")
        bgt:SetVertexColor(1.0, 1.0, 1.0)
        bgt:SetPoint("CENTER", ref, "CENTER", 0, 0)
        bgt:SetHeight(height*1.7)
        bgt:SetWidth(width*1.7)
        bgt:SetTexCoord(0, 1, 0, 1)
        bgt:SetBlendMode("BLEND")
        bgt:Hide()
        font = ref:CreateFontString("DHUD2_"..k.."_Text", "OVERLAY")
        font:ClearAllPoints()
        font:SetFontObject(GameFontHighlightSmall)
        font:SetPoint("CENTER", ref, "CENTER", 0,0)
        font:SetText("")
        font:SetJustifyH("CENTER")
        font:SetJustifyV("CENTER")
        font:SetWidth(width*2)
        font:SetHeight(height)
        ref:SetScript("OnEnter", function(this) 
                if not (this:IsVisible() and DHUD2.db.profile.show.auraTips) or (this:GetEffectiveAlpha() == 0) then
                    return
                end
                GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
                GameTooltip:SetTotem(this.slot)
            end )
        ref:SetScript("OnLeave", function() 
                GameTooltip:Hide()
            end )
        ref:SetScript("OnClick", function(this, mouseButton)
                DHUD2:Debug("Totem OnClick", mouseButton)
                if ( mouseButton == "LeftButton" ) then
            		DestroyTotem(this.slot)
            	end
            end )
        ref:EnableMouse(false)
        ref:Hide()
    end
    
    -- Icons
    for k, v in pairs(icons) do
        point, parent, relative, x, y, width, height, x0, x1, y0, y1, path = unpack(v)
        if string_find(k, "PvP") then         
            local faction = UnitFactionGroup("player")
            if faction == "Horde" then
                path = path[2]
            else
                path = path[1]
            end
        end
        ref = CreateFrame("Frame", "DHUD2_"..k, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        local bgt = ref:CreateTexture("DHUD2_"..k.."_Texture","BACKGROUND")
        bgt:SetTexture(path)
        bgt:ClearAllPoints()
        bgt:SetPoint("TOPLEFT", ref , "TOPLEFT", 0, 0)
        bgt:SetPoint("BOTTOMRIGHT", ref , "BOTTOMRIGHT", 0, 0)
        bgt:SetTexCoord(x0,x1,y0,y1)
        ref:Hide()
    end
    
    -- Target Auras
    for i = 1, 40 do
        point, parent, relative, x, y, width, height = unpack(tRA["TRA"..i])
        ref = CreateFrame("Button", "DHUD2_TRA"..i, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        local font = ref:CreateFontString("DHUD2_TRA"..i.."_Text", "OVERLAY")
        font:SetFontObject(GameFontHighlightSmall)
        font:SetText("")
        font:SetJustifyH("RIGHT")
        font:SetJustifyV("BOTTOM")
        font:SetWidth(width+1)
        --font:SetHeight(height-5)
        font:SetFont("FRIZQT__.TTF", width / 2.2, "OUTLINE")
        font:Show()
        font:ClearAllPoints()
        font:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        bgt = ref:CreateTexture("DHUD2_TRA"..i.."_Border", "ARTWORK")
        bgt:SetTexture("Interface\\Buttons\\UI-Debuff-Border")
        bgt:SetPoint("BOTTOM", ref, "BOTTOM", 0, 0)
        bgt:SetHeight(height)
        bgt:SetWidth(width)
        bgt:SetTexCoord(0, 1, 0, 1)
        ref:SetNormalTexture( "Interface\\Icons\\Ability_Druid_TravelForm" )
        ref:SetScript("OnEnter", function(this)
                if not (this:IsVisible() and DHUD2.db.profile.show.auraTips) then
                    return
                end
                GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
                if this.hasdebuff then
                    GameTooltip:SetUnitDebuff(this.unit, this.id)
                else
                    GameTooltip:SetUnitBuff(this.unit, this.id)
                end
            end )
        ref:SetScript("OnLeave", function() 
                GameTooltip:Hide()
            end )        
        ref:EnableMouse(true)
        ref:Hide()
    end
    
    for i = 1, 40 do
        point, parent, relative, x, y, width, height = unpack(tLA["TLA"..i])
        ref = CreateFrame("Button", "DHUD2_TLA"..i, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        local font = ref:CreateFontString("DHUD2_TLA"..i.."_Text", "ARTWORK")
        font:SetFontObject(GameFontHighlightSmall)
        font:SetText("")
        font:SetJustifyH("RIGHT")
        font:SetJustifyV("BOTTOM")
        font:SetWidth(width+1)
        --font:SetHeight(height-5)
        font:SetFont("FRIZQT__.TTF", width / 2.2, "OUTLINE")
        font:Show()
        font:ClearAllPoints()
        font:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        bgt = ref:CreateTexture("DHUD2_TLA"..i.."_Border","OVERLAY")
        bgt:SetTexture("Interface\\Buttons\\UI-Debuff-Border")
        bgt:SetPoint("BOTTOM", ref, "BOTTOM", 0, 0)
        bgt:SetHeight(height)
        bgt:SetWidth(width)
        bgt:SetTexCoord(0, 1, 0, 1)
        ref:SetNormalTexture( "Interface\\Icons\\Ability_Druid_TravelForm" )
        ref:SetScript("OnEnter", function(this)
                if not (this:IsVisible() and DHUD2.db.profile.show.auraTips) then
                    return
                end
                GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
                if this.hasdebuff then
                    GameTooltip:SetUnitDebuff(this.unit, this.id)
                else
                    GameTooltip:SetUnitBuff(this.unit, this.id)
                end
            end )
        ref:SetScript("OnLeave", function() 
                GameTooltip:Hide()
            end )        
        ref:EnableMouse(true)
        ref:Hide()
    end
    
    -- Player Buff
    for k, v in pairs(playBuff) do
        point, parent, relative, x, y, width, height = unpack(v)
        ref = CreateFrame("Button", "DHUD2_"..k, _G["DHUD2_"..parent])
        ref:ClearAllPoints()
        ref:SetPoint(point, _G["DHUD2_"..parent], relative, x, y)
        ref:SetHeight(height)
        ref:SetWidth(width)
        font = ref:CreateFontString("DHUD2_"..k.."_Text", "OVERLAY")
        font:SetFontObject(GameFontHighlightSmall)
        font:ClearAllPoints()
        font:SetPoint("CENTER", ref, "CENTER", 0,0)
        font:SetText("")
        font:SetJustifyH("CENTER")
        font:SetJustifyV("CENTER")
        font:SetWidth(width*2)
        --font:SetHeight(height)
        bgt = ref:CreateTexture("DHUD2_"..k.."_Border", "OVERLAY")
        bgt:SetTexture("Interface\\AddOns\\DHUD2\\textures\\serenity0")
        bgt:SetVertexColor(1.0, 1.0, 1.0)
        bgt:SetPoint("CENTER", ref, "CENTER", 0, 0)
        bgt:SetHeight(height*1.7)
        bgt:SetWidth(width*1.7)
        bgt:SetTexCoord(0, 1, 0, 1)
        bgt:SetBlendMode("BLEND")                
        ref:SetNormalTexture( "Interface\\AddOns\\DHUD2\\icon" )
        ref:SetScript("OnEnter", function(this) 
                if not (this:IsVisible() and DHUD2.db.profile.show.auraTips) or (this:GetEffectiveAlpha() == 0) then
                    return
                end
                GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
                GameTooltip:SetPlayerBuff(this.id)
            end )
        ref:SetScript("OnLeave", function() 
                GameTooltip:Hide()
            end )         
        ref:Hide()
    end
end
